-- Main application class

-- TODO: Quite a BLOB here, a bit of architecturing wouldn't hurt. This is unlikely to grow though.

import "Turbine";
import "Turbine.UI";
import "Turbine.UI.Lotro";

import "Equendil.UI.WindowManager";
import "Equendil.UI.Resources";
import "Equendil.UI.IconButton";
import "Equendil.UI.VirtualListBox";
import "Equendil.Utils.Maths";
import "Equendil.PluginMng.Shell";
import "Equendil.PluginMng.Plugin";
import "Equendil.PluginMng.Command";
import "Equendil.PluginMng.CommandConfigurator";
import "Equendil.PluginMng.Menu";
import "Equendil.PluginMng.MenuButton";
import "Equendil.PluginMng.KnownPlugins";

-- Default commands we always want the menu to have no matter what
local DefaultCommands = {
	-- { name = "Exit Boostrap", command = "/boostrap unload"	}, -- removed for now, no 'clean' way to exit the plugin
	{ name = "Bootstrap", command = "/bootstrap show", mode = Command.Mode.Always }
}

-- controls displayed by the plugins listbox
local PluginItem = class( Turbine.UI.Control );
function PluginItem:Constructor( plugin )
	Turbine.UI.Control.Constructor( self );

	self.plugin = plugin;
	
	self:SetSize( 490, 18 );
	self:SetBackColorBlendMode( Turbine.UI.BlendMode.AlphaBlend );
	
	-- auto load checkbox
	self.checkbox = Turbine.UI.Lotro.CheckBox();
	self.checkbox:SetParent( self );
	self.checkbox:SetSize( 18, 18 );
	self.checkbox:SetLeft( 10 );
	
	-- name, author, vesion labels ...
	self.labelName = Turbine.UI.Label();
	self.labelName:SetParent( self );
	self.labelName:SetParent( self );
	self.labelName:SetSize( 170, 18 );
	self.labelName:SetLeft( 40 );
	self.labelName:SetFont( Turbine.UI.Lotro.Font.Verdana14 );
	self.labelName:SetTextAlignment( Turbine.UI.ContentAlignment.MiddleLeft );
	self.labelName:SetForeColor( Equendil.UI.Color.TurbineWhite );
	self.labelName:SetMouseVisible( false );

	self.labelAuthor = Turbine.UI.Label();
	self.labelAuthor:SetParent( self );
	self.labelAuthor:SetSize( 170, 18 );
	self.labelAuthor:SetLeft( 220 );
	self.labelAuthor:SetFont( Turbine.UI.Lotro.Font.Verdana14 );
	self.labelAuthor:SetTextAlignment( Turbine.UI.ContentAlignment.MiddleLeft );
	self.labelAuthor:SetForeColor( Equendil.UI.Color.TurbineWhite );
	self.labelAuthor:SetMouseVisible( false );

	self.labelVersion = Turbine.UI.Label();
	self.labelVersion:SetParent( self );
	self.labelVersion:SetSize( 80, 18 );
	self.labelVersion:SetLeft( 400 );
	self.labelVersion:SetFont( Turbine.UI.Lotro.Font.Verdana14 );
	self.labelVersion:SetTextAlignment( Turbine.UI.ContentAlignment.MiddleLeft );
	self.labelVersion:SetForeColor( Equendil.UI.Color.TurbineWhite );
	self.labelVersion:SetMouseVisible( false );	

	-- Forward the checkbox status to the plugin
	self.checkbox.CheckedChanged = function( sender, args ) 
		self.plugin:SetAutoLoad( self.checkbox:IsChecked() );
	end
	
	-- register a callback with a plugin to update the display as needed
	self.pluginChangedAction = function() self:_Update(); end;
	self.plugin.Changed:Add( self.pluginChangedAction );
	
	self:_Update();
end

function PluginItem:_Update()
	self.checkbox:SetChecked( self.plugin:IsAutoLoad() );
	self.labelName:SetText( self.plugin:GetName() );
	self.labelAuthor:SetText( self.plugin:GetAuthor() );	
	self.labelVersion:SetText( self.plugin:GetVersion() );
	if self.plugin:IsLoaded() then
		-- Loaded plugins are displayed in green
		self.labelName:SetForeColor( Equendil.UI.Color.Green );
		self.labelAuthor:SetForeColor( Equendil.UI.Color.Green );
		self.labelVersion:SetForeColor( Equendil.UI.Color.Green );
	else
		self.labelName:SetForeColor( Equendil.UI.Color.TurbineWhite );
		self.labelAuthor:SetForeColor( Equendil.UI.Color.TurbineWhite );
		self.labelVersion:SetForeColor( Equendil.UI.Color.TurbineWhite );
	end
end

-- VirtualListBox can call this to set the selection status
function PluginItem:SetSelected( flag )
	if flag then
		self:SetBackColor( Turbine.UI.Color( 0.2, 0.9, 0.9, 1 ) );
	else
		self:SetBackColor( Turbine.UI.Color( 0, 0, 0, 0 ) );
	end
end

-- controls displayed in thec ommands listbox
local CommandItem = class( Turbine.UI.Control );
function CommandItem:Constructor( command, isDefault )
	Turbine.UI.Control.Constructor( self );

	self.command = command;
	self.isDefault = isDefault;
	
	self:SetSize( 190, 18 );
	self:SetBackColorBlendMode( Turbine.UI.BlendMode.AlphaBlend );	
	
	-- A label is really all we need to show here
	self.labelName = Turbine.UI.Label();
	self.labelName:SetParent( self );
	self.labelName:SetSize( 190, 18 );
	self.labelName:SetLeft( 0 );
	self.labelName:SetFont( Turbine.UI.Lotro.Font.Verdana14 );
	self.labelName:SetTextAlignment( Turbine.UI.ContentAlignment.MiddleLeft );
	self.labelName:SetForeColor( Equendil.UI.Color.TurbineWhite );
	self.labelName:SetMouseVisible( false );
	
	--  register a callback to update when a command changes
	self.commandChanged = function() self:_Update(); end
	self.command.Changed:Add( self.commandChanged );

	self:_Update();
end

function CommandItem:_Update()
	self.labelName:SetText( self.command:GetName() );
end

function CommandItem:SetSelected( flag )
	if flag then
		self:SetBackColor( Turbine.UI.Color( 0.2, 0.9, 0.9, 1 ) );
	else
		self:SetBackColor( Turbine.UI.Color( 0, 0, 0, 0 ) );
	end
end

-- Main application class
App = class();
function App:Constructor()
	
	-- Variables
	self.pluginItems = {}; -- plugins listbox elements.
	self.commands = {}; -- commands
	self.commandItems = {}; -- commands listbox elements
	
	self.persistentData = {}; -- stored data for plugins & commands
	self.firstTimeUse = true; -- whether or not this is the first time the plugin is launched, might be altered as data is loaded

	self.version = "?"; -- version of this plugin, set somewhere else with real data
	
	-- UI elements
	
	-- setup a window and center it
	self.window = Turbine.UI.Lotro.Window();
    self.window:SetSize( 575, 600 );
	self.window:SetPosition( 
		( Turbine.UI.Display:GetWidth() - self.window:GetWidth() ) / 2, 
		( Turbine.UI.Display:GetHeight() - self.window:GetHeight() ) /2 
	);	
	self.window:SetText( "Bootstrap - Plugin Manager" );
	self.window:SetParent( nil );
	Equendil.UI.WindowManager:ManageWindow( self.window ); -- makes this window managed for HUD toggle / ESC
	
	-- Add a background so it does not look too dull
	self.background = Equendil.UI.DecoratedContainer( Equendil.UI.Decoration.GrainyBlue );
	self.background:SetParent( self.window );
	self.background:SetSize( 512, 560 );
	self.background:SetPosition( 31, 30 );

	-- PLUGINS ELEMENTS
	
	-- Container for all controls related to plugins
	self.backgroundPlugins = Turbine.UI.Control();
	self.backgroundPlugins:SetParent( self.window );
	self.backgroundPlugins:SetSize( 530, 270 );
	self.backgroundPlugins:SetPosition( 22, 30 );
	
	-- listbox headers
	self.labelPluginsHeaderAutoLoad = Equendil.UI.CommonControl.LabelListHeading( "Auto" );
	self.labelPluginsHeaderAutoLoad:SetParent( self.backgroundPlugins );
	self.labelPluginsHeaderAutoLoad:SetSize( 40, 14 );
	self.labelPluginsHeaderAutoLoad:SetPosition( 10, 5 );
	
	self.labelPluginsHeaderName = Equendil.UI.CommonControl.LabelListHeading( "Plugin Name" );
	self.labelPluginsHeaderName:SetParent( self.backgroundPlugins );
	self.labelPluginsHeaderName:SetSize( 170, 14 );
	self.labelPluginsHeaderName:SetPosition( 50, 5 );
	
	self.labelPluginsHeaderAuthor = Equendil.UI.CommonControl.LabelListHeading( "Author" );
	self.labelPluginsHeaderAuthor:SetParent( self.backgroundPlugins );
	self.labelPluginsHeaderAuthor:SetSize( 170, 14 );
	self.labelPluginsHeaderAuthor:SetPosition( 230, 5 );

	self.labelPluginsHeaderVersion = Equendil.UI.CommonControl.LabelListHeading( "Version" );
	self.labelPluginsHeaderVersion:SetParent( self.backgroundPlugins );
	self.labelPluginsHeaderVersion:SetSize( 70, 14 );
	self.labelPluginsHeaderVersion:SetPosition( 410, 5 );

	-- borders around the plugins listbox
	self.outlinePlugins = Equendil.UI.DecoratedContainer( Equendil.UI.Decoration.RoundSilverBlue );
	self.outlinePlugins:SetParent( self.backgroundPlugins );
	self.outlinePlugins:SetSize( 520, 220 );
	self.outlinePlugins:SetPosition( 0, 20 );
	
	-- scrollbar for listbox
	self.scrollbarPlugins = Turbine.UI.Lotro.ScrollBar();
	self.scrollbarPlugins:SetOrientation( Turbine.UI.Orientation.Vertical );
	self.scrollbarPlugins:SetParent( self.backgroundPlugins );
	self.scrollbarPlugins:SetSize( 10, 200 );
	self.scrollbarPlugins:SetPosition( 500, 30 );
	
	-- Plugins listbox
	local getPluginItem = function( n, selection )
		local pluginItem = self.pluginItems[n];
		pluginItem:SetSelected( selection );
		return pluginItem;
	end
	self.listboxPlugins = Equendil.UI.VirtualListBox( getPluginItem );
	self.listboxPlugins:SetParent( self.backgroundPlugins );
	self.listboxPlugins:SetSize( 490, 200 );
	self.listboxPlugins:SetPosition( 10, 30 );
	self.listboxPlugins:SetItemHeight( 18 );
	self.listboxPlugins:SetVerticalScrollBar( self.scrollbarPlugins );
	self.listboxPlugins:SetSelectionMode( Equendil.UI.SelectionMode.SingleSelection );
	self.listboxPlugins:SetBackColor( Turbine.UI.Color( 0.1, 1, 1, 1 ) );
	self.listboxPlugins:SetBackColorBlendMode( Turbine.UI.BlendMode.Overlay );

	-- buttons for actions on plugins
	
	-- up/down buttons to order plugins
	self.buttonPluginUp = Equendil.UI.ButtonArrowUp20();
	self.buttonPluginUp:SetParent( self.backgroundPlugins );
	self.buttonPluginUp:SetPosition( 10, 243 );
	
	self.buttonPluginDown = Equendil.UI.ButtonArrowDown20();
	self.buttonPluginDown:SetParent( self.backgroundPlugins );
	self.buttonPluginDown:SetPosition( 30, 243 );

	-- "Load plugin" action
	self.buttonPluginLoad = Turbine.UI.Lotro.Button();
	self.buttonPluginLoad:SetParent( self.backgroundPlugins );
	self.buttonPluginLoad:SetSize( 80, 20 );
	self.buttonPluginLoad:SetPosition( 60, 243 );
	self.buttonPluginLoad:SetText( "Launch" );
	self.buttonPluginLoad:SetEnabled( false );

	-- "Add to Menu" action
	self.buttonPluginsAddCommand = Turbine.UI.Lotro.Button();
	self.buttonPluginsAddCommand:SetParent( self.backgroundPlugins );
	self.buttonPluginsAddCommand:SetSize( 120, 20 );
	self.buttonPluginsAddCommand:SetPosition( 150, 243 );
	self.buttonPluginsAddCommand:SetText( "Add to Menu" );
	self.buttonPluginsAddCommand:SetEnabled( false );

	-- "Refresh List" button
	self.buttonPluginsRefresh = Turbine.UI.Lotro.Button();
	self.buttonPluginsRefresh:SetParent( self.backgroundPlugins );
	self.buttonPluginsRefresh:SetSize( 100, 20 );
	self.buttonPluginsRefresh:SetPosition( 430, 243 );
	self.buttonPluginsRefresh:SetText( "Refresh List" );
	self.buttonPluginsRefresh.Click = function( sender, args ) self:RefreshPlugins(); end
	
	-- COMMANDS 
	
	-- container for all controls related to commands
	self.backgroundCommands = Turbine.UI.Control();
	self.backgroundCommands:SetParent( self.window );
	self.backgroundCommands:SetSize( 530, 270 );
	self.backgroundCommands:SetPosition( 22, 310 );
	
	-- headers
	self.labelCommandsHeader = Equendil.UI.CommonControl.LabelListHeading( "Menu Commands" );
	self.labelCommandsHeader:SetParent( self.backgroundCommands );
	self.labelCommandsHeader:SetSize( 200, 14 );
	self.labelCommandsHeader:SetPosition( 0, 5 );
	
	self.labelCommandsHeaderDetail = Equendil.UI.CommonControl.LabelListHeading( "Detail" );
	self.labelCommandsHeaderDetail:SetParent( self.backgroundCommands );
	self.labelCommandsHeaderDetail:SetSize( 270, 14 );
	self.labelCommandsHeaderDetail:SetPosition( 230, 5 );
	
	-- borders around the listbox
	self.outlineCommands = Equendil.UI.DecoratedContainer( Equendil.UI.Decoration.RoundSilverBlue );
	self.outlineCommands:SetParent( self.backgroundCommands );
	self.outlineCommands:SetSize( 220, 220 );
	self.outlineCommands:SetPosition( 0, 20 );
	
	-- scrollbar for the list box
	self.scrollbarCommands = Turbine.UI.Lotro.ScrollBar();
	self.scrollbarCommands:SetOrientation( Turbine.UI.Orientation.Vertical );
	self.scrollbarCommands:SetParent( self.backgroundCommands );
	self.scrollbarCommands:SetSize( 10, 200 );
	self.scrollbarCommands:SetPosition( 200, 30 );	
	
	-- Listbox for commands
	local getCommandItem = function( n, selection )
		local commandItem = self.commandItems[n];
		commandItem:SetSelected( selection );
		return commandItem;
	end
	self.listboxCommands = Equendil.UI.VirtualListBox( getCommandItem );
	self.listboxCommands:SetParent( self.backgroundCommands );
	self.listboxCommands:SetSize( 190, 200 );
	self.listboxCommands:SetPosition( 10, 30 );
	self.listboxCommands:SetItemHeight( 18 );
	self.listboxCommands:SetVerticalScrollBar( self.scrollbarCommands );
	self.listboxCommands:SetSelectionMode( Equendil.UI.SelectionMode.SingleSelection );
	self.listboxCommands:SetBackColor( Turbine.UI.Color( 0.1, 1, 1, 1 ) );
	self.listboxCommands:SetBackColorBlendMode( Turbine.UI.BlendMode.Overlay );
	
	-- up/down buttons to order commands within the menu
	self.buttonCommandUp = Equendil.UI.ButtonArrowUp20();
	self.buttonCommandUp:SetParent( self.backgroundCommands );
	self.buttonCommandUp:SetPosition( 0, 243 );
	
	self.buttonCommandDown = Equendil.UI.ButtonArrowDown20();
	self.buttonCommandDown:SetParent( self.backgroundCommands );
	self.buttonCommandDown:SetPosition( 20, 243 );
	
	-- "Remove command" action
	self.buttonCommandsRemove = Turbine.UI.Lotro.Button();
	self.buttonCommandsRemove:SetParent( self.backgroundCommands );
	self.buttonCommandsRemove:SetSize( 80, 20 );
	self.buttonCommandsRemove:SetPosition( 40, 243 );
	self.buttonCommandsRemove:SetText( "Remove" );
	
	-- "Save Menu" action
	self.buttonCommandsSave = Turbine.UI.Lotro.Button();
	self.buttonCommandsSave:SetParent( self.backgroundCommands );
	self.buttonCommandsSave:SetSize( 100, 20 );
	self.buttonCommandsSave:SetPosition( 130, 243 );
	self.buttonCommandsSave:SetText( "Save Menu" );
	
	-- borders & external controls to fill in details about a given command
	self.outlineCommandConfigurator = Equendil.UI.DecoratedContainer( Equendil.UI.Decoration.RoundSilverBlue );
	self.outlineCommandConfigurator:SetParent( self.backgroundCommands );
	self.outlineCommandConfigurator:SetSize( 300, 220 );
	self.outlineCommandConfigurator:SetPosition( 220, 20 );
	
	self.commandConfigurator = Equendil.PluginMng.CommandConfigurator();
	self.commandConfigurator:SetParent( self.backgroundCommands );
	self.commandConfigurator:SetSize( 280, 200 );
	self.commandConfigurator:SetPosition( 230, 30 );
	self.commandConfigurator:SetBackColor( Turbine.UI.Color( 0.1, 0.5, 1, 0.5 ) );
	self.commandConfigurator:SetBackColorBlendMode( Turbine.UI.BlendMode.AlphaBlend );

	-- "show menu" checkbox & label
	self.checkboxMenuButtonVisible = Turbine.UI.Lotro.CheckBox();
	self.checkboxMenuButtonVisible:SetParent( self.backgroundCommands );
	self.checkboxMenuButtonVisible:SetSize( 18, 18 );
	self.checkboxMenuButtonVisible:SetPosition( 290, 243 );
	self.checkboxMenuButtonVisible:SetChecked( true );

	self.labelMenuButtonVisible = Equendil.UI.CommonControl.LabelFieldHeading( "Show menu button" );
	self.labelMenuButtonVisible:SetParent( self.backgroundCommands );
	self.labelMenuButtonVisible:SetSize( 200, 18 );
	self.labelMenuButtonVisible:SetPosition( 310, 243 );
	
	-- ACTIONS

	-- action on plugins 
	self.listboxPlugins.SelectionChanged = function() self:_PluginsSelectionChanged() end
	self.buttonPluginUp.Clicked = function( button ) self:_PluginUp() end
	self.buttonPluginDown.Clicked = function( button ) self:_PluginDown() end
	self.buttonPluginsAddCommand.Click = function( sender, args ) self:_AddCommand() end
	self.buttonPluginLoad.Click = function( sender, args ) self:_LoadSelectedPlugin() end

	-- action on commands
	self.listboxCommands.SelectionChanged = function( sender, args ) self:_CommandsSelectionChanged() end
	self.buttonCommandUp.Clicked = function( button ) self:_CommandUp() end
	self.buttonCommandDown.Clicked = function( button ) self:_CommandDown() end
	self.buttonCommandsRemove.Click = function( sender, args ) self:_RemoveSelectedCommand() end
	self.buttonCommandsSave.Click = function( sender, args ) self:SaveCommands(); end

	-- show menu option
	self.checkboxMenuButtonVisible.CheckedChanged = function( sender, args ) self:_SetMenuButtonVisibility() end;

	-- define an action for use whenever a plugin is set to autoload
	self.actionSaveAutoLoad = function() self:SaveAutoLoad(); end	
	
	-- setup menu & start button
	self.menu = Equendil.PluginMng.Menu();
	
	self.menuButton = Equendil.PluginMng.MenuButton( self.menu );
	self.menuButton:SetParent( nil );
	self.menuButton:SetVisible( true );
	self.menuButton.Changed = function() self:SaveMenu(); end

	-- create a Shell object so we can execute plugin commands
	self.shell = Equendil.PluginMng.Shell();
	self.shell:Hook(); -- hook Turbine.Shell to catch other plugins' commands

	-- register a function to unhook the shell when this plugin is about to exit
	Plugins["Bootstrap"].Unload = function()
		self.shell:Unhook(); -- we really need to clear our hook before we go
	end	
	
	-- Finally, get stored data, build the list of plugins (and commands)
	self:Load();
	self:FetchPlugins();
	
	-- Output something in the chat so the user knows the plugin has started
	Turbine.Shell.WriteLine( "Boostrap version " .. self.version .. " started." );
	
	-- Load all plugins set to auto
	self:AutoLoadPlugins();

	-- finally, go visible if it's the first time the plugin is used
	if self.firstTimeUse then self:Show() end
end

-- handle selection of a plugin
function App:_PluginsSelectionChanged()
	local n = self.listboxPlugins:GetSelectedIndex();
	self.buttonPluginLoad:SetEnabled( n ~= -1 );
	self.buttonPluginsAddCommand:SetEnabled( n ~= -1 );
end
	
-- move a plugin up the list of plugins	
function App:_PluginUp()
	local n = self.listboxPlugins:GetSelectedIndex();
	if n > 1 then -- check we have something selected and it's not first already
		self.pluginItems[n-1], self.pluginItems[n] = self.pluginItems[n], self.pluginItems[n-1]; -- swap 
		self.listboxPlugins:Select( n - 1 );
		self.listboxPlugins:Refresh();
		self.listboxPlugins:MakeVisible( n - 1 );
		
		self:SavePlugins(); -- we save the list of plugins automatically
	end
end

-- move a plugin down the list of plugins	
function App:_PluginDown()
	local n = self.listboxPlugins:GetSelectedIndex();
	if n ~= -1 and n ~= self.listboxPlugins:GetItemCount() then 	-- check if we have something selected and it's not last of list already
		self.pluginItems[n+1], self.pluginItems[n] = self.pluginItems[n], self.pluginItems[n + 1 ]; -- swap
		self.listboxPlugins:Select( n + 1 );
		self.listboxPlugins:Refresh();
		self.listboxPlugins:MakeVisible( n + 1 );
		
		self:SavePlugins(); -- we save the list of plugins automatically
	end
end

-- load the currently selected plugin
function App:_LoadSelectedPlugin()
	local n = self.listboxPlugins:GetSelectedIndex();
	if n ~= -1 then
		self.pluginItems[n].plugin:Load();
	end
end

-- handle selection of a command
function App:_CommandsSelectionChanged()
	local n = self.listboxCommands:GetSelectedIndex();
	self.buttonCommandsRemove:SetEnabled( n > #DefaultCommands );
	if n ~= -1 then -- check we have something selected
		-- setup the external control with that command
		self.commandConfigurator:SetCommand( self.commands[n] );
		self.commandConfigurator:SetReadOnly( self.commandItems[n].isDefault );
	end
end

-- move a command up the list of commands
function App:_CommandUp()
	local n = self.listboxCommands:GetSelectedIndex();
	if n > #DefaultCommands + 1 then -- make sure we have something selected and it's not a default command (they stay at the top)
		self.commandItems[n-1], self.commandItems[n] = self.commandItems[n], self.commandItems[n-1]; -- swap 
		self.commands[n-1], self.commands[n] = self.commands[n], self.commands[n-1]; -- swap
		self.listboxCommands:Select( n - 1 );
		self.listboxCommands:Refresh();
		self.listboxCommands:MakeVisible( n - 1 );
		
		self.menu:Set( self.commands ); -- feed the 'new' command list to the menu again
	end
end

-- move a command down the list of commands
function App:_CommandDown()
	local n = self.listboxCommands:GetSelectedIndex();
	if n > #DefaultCommands and n ~= self.listboxCommands:GetItemCount() then -- make sure we have something selected, not at the bottom already, not a default command (they stay at the top)
		self.commandItems[n+1], self.commandItems[n] = self.commandItems[n], self.commandItems[n + 1 ]; -- swap
		self.commands[n+1], self.commands[n] = self.commands[n], self.commands[n+1 ];
		self.listboxCommands:Select( n + 1 );
		self.listboxCommands:Refresh();
		self.listboxCommands:MakeVisible( n + 1 );
		
		self.menu:Set( self.commands ); -- feed the 'new' command list to the menu again
	end
end

-- remove the selected command from the menu
function App:_RemoveSelectedCommand()
	local n = self.listboxCommands:GetSelectedIndex();
	if n > #DefaultCommands then -- make sure we have something selected and it's not a default command
		table.remove( self.commands, n )
		table.remove( self.commandItems, n );
		self.listboxCommands:SetItemCount( #self.commandItems );
		self.listboxCommands:Refresh();
		
		self.menu:Set( self.commands ); -- feed the 'new' command list to the menu
	end
end	

-- set the menu button visibility according to the checkbox status
function App:_SetMenuButtonVisibility()
	self.menuButton:SetVisible( self.checkboxMenuButtonVisible:IsChecked() );
	self:SaveMenu();
end

-- return index & PluginItem object in he list of plugins from its name. (-1, nil) if not found
function App:_FindPlugin( name )
	for i, pluginItem in ipairs( self.pluginItems ) do
		if pluginItem.plugin:GetName() == name then
			return i, pluginItem.plugin;
		end
	end
	
	return -1, nil;
end

-- return stored index and data of a given plugin. (-1, nil) if not found
function App:_FindStoredPlugin( name )
	if self.persistentData == nil or self.persistentData.plugins == nil then
		return -1, nil;
	end
	
	for i, storedPlugin in ipairs( self.persistentData.plugins ) do
		if  storedPlugin.name == name then
			return i, storedPlugin;
		end
	end
	
	return -1, nil;
end

-- returns whether a plugin is in the auto load table
function App:_IsAutoLoad( name )
	if self.persistentData == nil or self.persistentData.autoLoad == nil then
		return false;
	end
	
	for i, autoLoadPluginName in ipairs( self.persistentData.autoLoad ) do
		if autoLoadPluginName == name then
			return true;
		end
	end
end

-- get the list of existing plugins, and set data of associated commands
function App:FetchPlugins()
	for i, pluginItems in ipairs( self.pluginItems ) do
		pluginItems.plugin.AutoLoadChanged:Remove( self.actionSaveAutoLoad );
	end
	self.pluginItems = {};
	local turbinePlugins = Turbine.PluginManager.GetAvailablePlugins();
	
	-- update the list of plugins, order "known" plugins and set their autoload status from stored data if available,
	local knownPlugins = {}
	local unknownPlugins = {};
	for k, turbinePlugin in pairs( turbinePlugins ) do
		if turbinePlugin.Name ~= "Bootstrap" then -- do *not* show our own plugin, ever.
			local plugin = Plugin( turbinePlugin );
			local n, storedPlugin = self:_FindStoredPlugin( plugin:GetName() );
			if storedPlugin ~= nil then -- we have info about this plugin, add it in the "known" plugin table, in its saved index position
				plugin:SetAutoLoad( self:_IsAutoLoad( plugin:GetName() ) );
				knownPlugins[n] = PluginItem( plugin );
			else -- plugins we have no info for get added to a different table for merging below 
				table.insert( unknownPlugins, PluginItem( plugin ) );
			end
			plugin.AutoLoadChanged:Add( self.actionSaveAutoLoad ); -- register an action to save auto load status of plugins when that changes
		else
			self.version = turbinePlugin.Version; -- extract version
		end
	end
	
	-- merge 'known' and 'unknown' plugins into a single, ordered table (unknown plugins at the end)
	-- the table of 'known' plugins might have "holes" in it, so we have to use pairs() rather than ipairs()
	-- assuming pairs() returns items in order of their index here
	for i, pluginItem in pairs( knownPlugins ) do 
		table.insert( self.pluginItems, pluginItem );
	end
	for i, pluginItem in ipairs(unknownPlugins) do
		table.insert( self.pluginItems, pluginItem );
	end

	-- and refresh the list of plugins
	self.listboxPlugins:SetItemCount( #self.pluginItems );
	self.listboxPlugins:Refresh();
	
	-- create commands
	self:_CreateDefaultCommands();
	self:_AddSavedCommands();
	
	-- refresh the commands listbox & menu
	self.listboxCommands:SetItemCount( #self.commandItems );
	self.listboxCommands:Refresh();
	self.menu:Set( self.commands );
end

-- add commands that we have stored
function App:_AddSavedCommands()
	if self.persistentData == nil or self.persistentData.commands == nil then	
		return;
	end

	for k, commandData in ipairs( self.persistentData.commands ) do
		local i, plugin = self:_FindPlugin( commandData.pluginName );
		if plugin ~= nil then
			-- found that plugin, add command
			local command = Command( self.shell );
			command:SetPlugin( plugin );
			command:SetName( commandData.name );
			command:SetShellCommand( commandData.shellCommand );
			command:SetMode( commandData.mode );
			table.insert( self.commands, command );
			table.insert( self.commandItems, CommandItem( command, false ) );
			self.menu:Set( self.commands );
		end
	end
end

-- create a command from a template. plugin = plugin to attach to the command
function App:_CreateCommandFromTemplate( template, plugin )
	local command = Command( self.shell );
	command:SetName( template.name );
	command:SetMode( template.mode );
	command:SetShellCommand( template.command );
	command:SetPlugin( plugin );
	return command;
end

-- reset commands and create a bunch of commands to use by default
function App:_CreateDefaultCommands()
	-- remove callbacks & clear tables
	for k, v in pairs( self.commands ) do 
		v.Changed:Remove( self.commandChanged );
	end

	self.commands = {};
	self.commandItems = {};

	-- create commands from default templates
	for i, commandTemplate in ipairs( DefaultCommands ) do
		local command = self:_CreateCommandFromTemplate( commandTemplate, nil );
		self.commands[i] = command;
		
		self.commandItems[i] = CommandItem( command, true );
	end
end

-- Find a template for a given plugin name if any
function App:_FindKnownPluginsCommandTemplate( pluginName )
	for k, template in pairs( KnownPlugins.CommandTemplates ) do
		if k == pluginName then
			return template;
		end
	end
	
	return nil;
end

-- add a new command for the selected plugin. If we have a template for it, we use that, if not, we setup a default command
function App:_AddCommand()
	-- check that we have something selected
	local n = self.listboxPlugins:GetSelectedIndex();
	if n == -1 then return end
	
	local plugin = self.pluginItems[n].plugin;
	local command = nil;

	-- we're 'smart'  here, if we have a default command template for this plugin, add that, otherwise build a dummy one
	local template = self:_FindKnownPluginsCommandTemplate( plugin:GetName() );
	if template ~= nil then
		-- we've got a default command template for this plugin, add it
		command = self:_CreateCommandFromTemplate( template, plugin );
	else
		command = Equendil.PluginMng.Command( self.shell );
		command:SetShellCommand( "/do something" );
		command:SetName( plugin:GetName() );
		command:SetPlugin( plugin );
		if plugin:GetScriptState() ~= "Default" then
			command:SetMode( Command.Mode.LoadOnly ); -- non default script state, can't run command on this plugin
		else
			command:SetMode( Command.Mode.LoadOrRun );
		end
	end
	
	-- add the new command to out tables
	table.insert( self.commands, command );
	table.insert( self.commandItems, CommandItem( command, false ) );
	
	-- refresh the list of commands
	self.listboxCommands:SetItemCount( #self.commandItems );
	self.listboxCommands:Refresh();
	
	-- and update the menu
	self.menu:Set( self.commands );
end

-- equivalent to /plugins refresh + /plugins list in turbine's shell
function App:RefreshPlugins()
	Turbine.PluginManager.RefreshAvailablePlugins();
	self.commandConfigurator:SetCommand( nil );
	self:FetchPlugins();
end

-- Load all plugins set to "auto" load
function App:AutoLoadPlugins()
	for i, pluginItem in ipairs( self.pluginItems ) do
		if pluginItem.plugin:IsAutoLoad() then
			pluginItem.plugin:Load();
			Turbine.Shell.WriteLine( "Bootstrap: Auto-loading " .. pluginItem.plugin:GetName() );
		end
	end
end

-- show/hide the window
function App:Show( flag )
	self.window:SetVisible( flag );
	self.window:Activate();
	self.listboxPlugins:Refresh(); -- refresh the listbox to check for loaded states 
end

-- Nuke all plugins including self
function App:Nuke()
	-- TODO: no way to do it properly atm
end


-- PERSISTENT STORAGE

function App:SerializeCommands()
	local data = {};

	-- seriazlize commands
	data.commands = {};
	for i, command in ipairs( self.commands ) do
		if i > #DefaultCommands then -- skip default commands
			local commandData = {};
			commandData.name = command:GetName();
			if command:GetPlugin() ~= nil then
				commandData.pluginName = command:GetPlugin():GetName();
			end
			commandData.shellCommand = command:GetShellCommand();
			commandData.mode = command:GetMode();
			table.insert( data.commands, commandData );
		end
	end
	
	return data;
end

function App:SerializePlugins()
	local data = {};
	
	-- serialize plugins
	data.plugins = {};
	for i, pluginItem in ipairs( self.pluginItems ) do
		data.plugins[i] = {};
		data.plugins[i].name = pluginItem.plugin:GetName();
		-- data.plugins[i].autoload = pluginItem.plugin:IsAutoLoad();
	end
	
	return data;
end

function App:SerializeAutoLoad()
	local data = {};
	
	data.autoLoad = {};
	for i, pluginItem in ipairs( self.pluginItems ) do 
		if pluginItem.plugin:IsAutoLoad() then
			table.insert( data.autoLoad, pluginItem.plugin:GetName() );
		end
	end
	
	return data;
end

function App:SerializeMenu()
	local data = {};

	-- serialize menu position & status of menu button
	data.buttonPosition = {};
	data.buttonPosition.left = self.menuButton:GetLeft();
	data.buttonPosition.top = self.menuButton:GetTop();
	data.buttonVisible = self.menuButton:IsVisible();
	
	return data;
end
	
function App:DeserializeMenu( data )
	-- position the menu button from stored data & set its visilibity
	if data.buttonPosition ~= nil then
		local left = Equendil.Utils.Maths.constrain( 0, data.buttonPosition.left, Turbine.UI.Display.GetWidth() - self.menuButton:GetWidth() );
		local top = Equendil.Utils.Maths.constrain( 10, data.buttonPosition.top, Turbine.UI.Display.GetHeight() - self.menuButton:GetHeight() );
		self.menuButton:SetPosition( left, top );
		
		if not data.buttonVisible then
			self.menuButton:SetVisible( false );
		end
	end
end
	
-- load any data we might have stored
function App:Load()
	self.persistentData = {};
	-- callbacks handling the data in defered loading

	local pluginsDataLoader = function( data ) self.persistentData.plugins = data.plugins; end
	local commandsDataLoader = function( data )	self.persistentData.commands = data.commands; end
	local autoLoadDataLoader = function( data ) self.persistentData.autoLoad = data.autoLoad; end
	local menuDataLoader = function( data ) self:DeserializeMenu( data ); end
	
	local dataLoader = function( loader )
		return function( data ) 
			if data ~= nil then 
				self.firstTimeUse = false;		
				loader( data );
			end
		end
	end
	
	-- Get the data, which we might get either as a return value or through the callback, we get it through the callback no matter what though, so it's handled there
	Turbine.PluginData.Load( Turbine.DataScope.Character, "EQ_Bootstrap_autoload", dataLoader( autoLoadDataLoader ) );
	Turbine.PluginData.Load( Turbine.DataScope.Account, "EQ_Bootstrap_commands",  dataLoader( commandsDataLoader ) );
	Turbine.PluginData.Load( Turbine.DataScope.Account, "EQ_Bootstrap_plugins",  dataLoader( pluginsDataLoader ) );
	Turbine.PluginData.Load( Turbine.DataScope.Character, "EQ_Bootstrap_menu", dataLoader( menuDataLoader ) );
end

function App:SaveAutoLoad()
	function dataSaved() end -- nothing to do here
	
	local data = self:SerializeAutoLoad();
	Turbine.PluginData.Save( Turbine.DataScope.Character, "EQ_Bootstrap_autoload", data, dataSaved );
	
	self.persistentData.autoLoad = data.autoLoad;
end

function App:SavePlugins()
	function dataSaved() end -- nothing to do here
	
	local data = self:SerializePlugins();
	Turbine.PluginData.Save( Turbine.DataScope.Account, "EQ_Bootstrap_plugins", data, dataSaved );
	
	self.persistentData.plugins = data.plugins;
end

function App:SaveCommands()
	function dataSaved() end -- nothing to do here
	
	local data = self:SerializeCommands();
	Turbine.PluginData.Save( Turbine.DataScope.Account, "EQ_Bootstrap_commands", data, dataSaved );
	
	self.persistentData.commands = data.commands;
end

function App:SaveMenu()
	function dataSaved() end -- nothing to do here
	
	local menuData = self:SerializeMenu();
	Turbine.PluginData.Save( Turbine.DataScope.Character, "EQ_Bootstrap_menu", menuData, dataSaved );
end