-- CommandConfigurator class. User controls for configuration of a command

import "Equendil.UI.CommonControls";
import "Equendil.UI.DropDownMenu";
import "Equendil.UI.TextBox";

import "Equendil.PluginMng.Command";

CommandConfigurator = class( Turbine.UI.Control );
function CommandConfigurator:Constructor()
	Turbine.UI.Control.Constructor( self );
	
	self.command = nil;
	
	-- name
	self.labelName = Equendil.UI.CommonControl.LabelFieldHeading( "Name" );
	self.labelName:SetParent( self );
	self.labelName:SetPosition( 10, 10 );
	self.labelName:SetSize( 80, 20 );
	
	self.textboxName = Equendil.UI.TextBox();
	self.textboxName:SetParent( self );
	self.textboxName:SetPosition( 100, 10 );
	self.textboxName:SetSize( 170, 20 );
	self.textboxName:SetFont( Turbine.UI.Lotro.Font.Verdana14 );
	self.textboxName:SetMultiline( false );
	self.textboxName.TextChanged = function()
		if self.command ~= nil then
			self.command:SetName( self.textboxName:GetText() );
		end
	end
	
	-- plugin
	self.labelPlugin = Equendil.UI.CommonControl.LabelFieldHeading( "Plugin" );
	self.labelPlugin:SetParent( self );
	self.labelPlugin:SetPosition( 10, 40 );
	self.labelPlugin:SetSize( 80, 20 );
	
	self.labelPluginName = Turbine.UI.Label();
	self.labelPluginName:SetParent( self );
	self.labelPluginName:SetSize( 170, 20 );
	self.labelPluginName:SetPosition( 100, 40 );
	self.labelPluginName:SetFont( Turbine.UI.Lotro.Font.TrajanPro16 );
	self.labelPluginName:SetTextAlignment( Turbine.UI.ContentAlignment.MiddleLeft );
	self.labelPluginName:SetForeColor( Equendil.UI.Color.White );
	self.labelPluginName:SetFontStyle( Turbine.UI.FontStyle.Outline );
	self.labelPluginName:SetMouseVisible( false );
	
	-- mode
	self.labelMode = Equendil.UI.CommonControl.LabelFieldHeading( "Mode" );
	self.labelMode:SetParent( self );
	self.labelMode:SetPosition( 10, 70 );
	self.labelMode:SetSize( 80, 20 );
	
	local modes = {};
	-- modes[ Equendil.PluginMng.Command.Mode.Always ] = "Always run";
	modes[ Command.Mode.LoadAndRun ] = "Load then Run";
	modes[ Command.Mode.LoadOrRun ] = "Load or Run";
	modes[ Command.Mode.RunOnly ] = "Run Only";
	self.menuMode = Equendil.UI.DropDownMenu( modes, nil );
	self.menuMode:SetParent( self );
	self.menuMode:SetSize( 170, 20 );
	self.menuMode:SetPosition( 100, 70 );
	self.menuMode.Selection = function( key )
		self.command:SetMode( key );
	end
	
	-- shell command
	self.labelCommand = Equendil.UI.CommonControl.LabelFieldHeading( "Command" );
	self.labelCommand:SetParent( self );
	self.labelCommand:SetPosition( 10, 100 );
	self.labelCommand:SetSize( 80, 20 );
	
	self.textboxCommand = Equendil.UI.TextBox();
	self.textboxCommand:SetParent( self );
	self.textboxCommand:SetSize( 170, 20 );
	self.textboxCommand:SetPosition( 100, 100 );
	self.textboxCommand:SetFont( Turbine.UI.Lotro.Font.Verdana14 );
	self.textboxCommand:SetMultiline( false );
	self.textboxCommand.TextChanged = function()
		if self.command ~= nil then
			self.command:SetShellCommand( self.textboxCommand:GetText() );
		end
	end	
	
	self:SetReadOnly( true );
	
	self:Update();
	
	-- no need for a callback atm
	-- self.commandChanged = function() self:Update(); end 
end

function CommandConfigurator:SetCommand( command )
--[[	no need for a callback atm
	if self.command ~= nil then
		self.command.Changed:Remove( self.commandChanged );
	end
--]]
	self.command = command;
	self:SetReadOnly( self.command == nil );
--[[	no need for a callback atm
	if self.command ~= nil then
		self.command.Changed:Add( self.commandChanged ); 
	end
--]]
	self:Update();
end

function CommandConfigurator:Update()
	if self.command == nil then
		self.textboxName:SetText( "" );
		self.labelPluginName:SetText( "" );
		self.menuMode:SetCurrentKey( nil );
		self.textboxCommand:SetText( "" );
	else
		self.textboxName:SetText( self.command:GetName() );
		if self.command:GetPlugin() ~= nil then
			self.labelPluginName:SetText( self.command:GetPlugin():GetName() );
		else
			self.labelPluginName:SetText( "" );
		end
		self.menuMode:SetCurrentKey( self.command:GetMode() );
		self.textboxCommand:SetText( self.command:GetShellCommand() );
		if self.command:GetMode() == Command.Mode.LoadOnly then
			-- if the command is set to "LoadOnly" mode, we don't really want the user to edit the command field
			self.textboxCommand:SetReadOnly( true );
			self.textboxCommand:SetText( "Load Only" );
			self.textboxCommand:SetForeColor( Turbine.UI.Color( 1, 1, 0, 0 ) );
		else
			self.textboxCommand:SetForeColor( Equendil.UI.Color.TurbineWhite );
			self.textboxCommand:SetReadOnly( false );
		end
	end
end

function CommandConfigurator:SetReadOnly( flag )
	self.textboxName:SetReadOnly( flag and self.command );
	self.textboxCommand:SetReadOnly( flag or self.command == nil or self.command:GetMode() == Command.Mode.LoadOnly );
	self.menuMode:SetReadOnly( flag or self.command == nil or self.command:GetMode() == Command.Mode.LoadOnly );
end