-- MenuButton class as a minimum user interface to toggle the menu.
-- By default, the button is positioned to the bottom left corner of the display

import "Turbine.UI";

import "Equendil.UI.DragBar";

MenuButton = class( Turbine.UI.Window );
function MenuButton:Constructor( menu )
	Turbine.UI.Window.Constructor( self );
	
	self.Changed = Equendil.Utils.Event(); -- Event called without parameters if position changes
	
	self.menu = menu;
	-- self.menuShowing = false;
	
	self:SetOpacity( 0.3 );
	self:SetSize( 32, 32 );
	self:SetPosition( 0, Turbine.UI.Display:GetHeight() - 32 ); -- we go to the bottom / left of the display by default
	self:SetVisible( true );
	self.Close = function() return false; end -- we don't want this 'window' to close ever
	Equendil.UI.WindowManager:ManageWindow( self ); -- for escape/display toggle handling
	
	-- attach a drag bar to it for repositioning (CTRL+\)
	self.dragBar = Equendil.UI.DragBar( self );
	self.dragBar.Repositioned = function() self.Changed(); end
	Equendil.UI.WindowManager:ManageDragBar( self.dragBar ); -- the window manager handles will handle visilbity 
	
	self.menuButton = Equendil.UI.ButtonMenuButton();
	self.menuButton:SetBlendMode( Turbine.UI.BlendMode.Normal );
	self.menuButton:SetParent( self );
	
	
	self.menuButton.Clicked = function() -- TODO: handle loss of focus of menu -> going invisible while in menuShowing = true state
		if not self.menu:IsVisible() then
			local left, top = 0, 0;
			if self:GetLeft() > Turbine.UI.Display.GetWidth() / 2 then
				left = self:GetLeft() - self.menu:GetWidth() + self:GetWidth(); 
			else
				left = self:GetLeft();
			end
			
			if self:GetTop() > Turbine.UI.Display.GetHeight() / 2 then
				top = self:GetTop(); -- menu will handle its actual position from its bottom
			else
				top = self:GetTop() + self:GetHeight();
			end
			self.menu:SetPosition( left, top );
		end
		self.menu:Show( not self.menu:IsVisible() );
	end
	
	-- We go transparent a bit unless the mouse is over the button
	self.menuButton.Hovering = function( flag ) 
		if flag then
			self:SetOpacity( 1.0 );
		else
			self:SetOpacity( 0.3 );
		end
	end
end