-- Plugin class, encapsulates Turbine's plugin data and manages Loading/Unloading

import "Equendil.Utils.WeakEvent";

Plugin = class( Turbine.Object )
function Plugin:Constructor( turbinePlugin )
	self.Changed = Equendil.Utils.WeakEvent(); -- Fired whenever an attribute changes. No parameter
	self.AutoLoadChanged = Equendil.Utils.WeakEvent(); -- Fired whenever the auto load status of the plugin has changed 
		
	self.data = turbinePlugin;
	self.autoload = false;
	self.loaded = false;
end

function Plugin:GetName() return self.data.Name end
function Plugin:GetAuthor()	return self.data.Author end
function Plugin:GetVersion() return self.data.Version end
function Plugin:IsAutoLoad() return self.autoload end
function Plugin:GetScriptState() return self.data.ScriptState end

-- Check if plugin is loaded,. 
function Plugin:_CheckLoaded()
	-- No better way than to travel the list of all loaded plugins if we want live status, so we do that.
	local loaded = false;
	local turbinePlugins = Turbine.PluginManager.GetLoadedPlugins();
	for k, turbinePlugin in pairs( turbinePlugins ) do
		if turbinePlugin.Name == self.data.Name then
			loaded = true;
			break;
		end
	end

	-- if state has changed, fire an event
	if loaded ~= self.loaded then
		self.loaded = loaded;
		self.Changed();
	end
end

function Plugin:IsLoaded() 
	self:_CheckLoaded();
	return self.loaded;
end

function Plugin:SetAutoLoad( flag )
	if self.autoload ~= flag then
		self.autoload = flag;
		self.Changed();
		self.AutoLoadChanged();
	end
end

function Plugin:Load()
	Turbine.PluginManager.LoadPlugin( self.data.Name );
	self:_CheckLoaded();
end

function Plugin:Unload()
	if self.data.ScriptState ~= "Default" then
		Turbine.PluginManager.UnloadScriptState( self.data.ScriptState );
	end
	self:_CheckLoaded();
end
