-- The Shell class hooks Turbine.Shell so as to catch new plugin commands for execution.

Shell = class( Turbine.Object );
function Shell:Constructor()
	Turbine.Object.Constructor( self );

	-- former Turbine.Shell functions that we replaced stored here for later use
	self.oldAddCommand = nil;
	self.oldRemoveCommand = nil;
	
	self.commands = {}; -- we'll store all the commands added to the shell here
end

-- Hook to intercept commands added to Turbine Shell. Will only work for plugins in the same script state	
function Shell:Hook()
	-- save up former calls
	self.oldAddCommand = Turbine.Shell.AddCommand;
	self.oldRemoveCommand = Turbine.Shell.RemoveCommand;
	
	-- and hook up our owns
	Turbine.Shell.AddCommand = function( names, shellCommand )
		local numRegistered = self.oldAddCommand( names, shellCommand );
		self.commands[shellCommand] = names; -- using the shellCommand itself as key for easy removal
	end
	
	Turbine.Shell.RemoveCommand = function( shellCommand )
		self.oldRemoveCommand( shellCommand );
		self.commands[command] = nil;
	end
end

-- Remove the hook cleanly
function Shell:Unhook()
	if self.oldAddCommand ~= nil then -- better make sure to avoid 'accidents'
		Turbine.Shell.AddCommand = self.oldAddCommand;
	end
	
	if self.oldRemoveCommand ~= nil then -- as above
		Turbine.Shell.RemoveCommand = self.oldRemoveCommand;
	end
end

-- Execute a shell command if we know about it
function Shell:ExecuteShellCommand( commandString )
	-- Find the command name in the command string
	local beg, last = string.find( commandString, "/%a+" ); -- match a '/' followed by letters until a space character
	if beg == nil then
		-- pattern matching failed, output an error
		Turbine.Shell.WriteLine( "Bootstrap: malformed command called: " .. commandString );
		return;
	end

	-- Extract that name and find a command with that name
	local commandName = string.sub( commandString, beg + 1, last ); -- extract command name (ie, without '/' and ending space character)
	local command = self:_FindCommand( commandName );
	if command == nil then
		-- no command found, output an error
		Turbine.Shell.WriteLine( "Bootstrap: unknown command: " .. commandString );
		return;
	end

	-- extract arguments
	local args = string.sub( commandString, last + 2 ); -- extract arguments

	-- and run the command
	command:Execute( commandName, args ); -- and execute the command  (NOTE: Turbine's documentation is missing the first parameter )
end

-- Find a command from its name
function Shell:_FindCommand( commandName )
	for command, names in pairs( self.commands ) do
		for name in string.gmatch(names, "%a+" ) do
			if string.lower( name ) == string.lower( commandName ) then
				return command; -- found it !
			end
		end
	end
	
	return nil; -- not found
end

