import "Turbine.UI";

-- resources ID within game files
local rs_dragbar_normal = 0x41007df9; -- height = 20
local rs_dragbar_depressed = 0x41007dfa; -- height = 20
local rs_highlight = { top = 0x4110CD4A, right = 0x4110CD4C, bottom = 0x4110CD4D, left = 0x4110CD4B }; -- highlight box

DragBar = class( Turbine.UI.Window );
function DragBar:Constructor( target )
	Turbine.UI.Window.Constructor( self );
	
	self.Repositioned = Equendil.Utils.Event();
	
	-- init instance variables
	self.target = target; -- window to attach to
	self.dragging = false; -- dragging state

	-- setup the bar window and background
	self:SetVisible( false ); -- go invisible by default
	self:SetSize( target:GetWidth(), 20 );
	self:SetPosition( target:GetLeft(), target:GetTop() - self:GetHeight() );
	self:SetBackground( rs_dragbar_normal );

	-- window & control used to highlight the target window
	self.highlightBox = Turbine.UI.Window();
	self.highlightBox:SetParent( nil );
	self.highlightBox:SetSize( self.target:GetSize() );
	self.highlightBox:SetPosition( self.target:GetPosition() );
	self.highlightBoxDecoration = Equendil.UI.DecoratedContainer( rs_highlight );
	self.highlightBoxDecoration:SetParent( self.highlightBox );
	self.highlightBoxDecoration:SetSize( self.target:GetSize() );
	
	-- add a label with the name of the target window
	self.label = Turbine.UI.Label();
	self.label:SetParent( self );
	self.label:SetFont( Turbine.UI.Lotro.Font.Verdana12 );
	self.label:SetFontStyle( Turbine.UI.FontStyle.Outline );
	self.label:SetForeColor( Turbine.UI.Color( 1, 0.9, 0.9, 0.9 ) );
	self.label:SetOutlineColor( Turbine.UI.Color( 1, 0, 0, 0 )  );
	self.label:SetTextAlignment( Turbine.UI.ContentAlignment.MiddleCenter );
	self.label:SetSize( self:GetSize() );
	self.label:SetText( target:GetText() );
	self.label:SetMouseVisible(false); -- the label is pass through
	
	-- setup callbacks
	self.MouseDown = function( sender, args ) 	self:_BeginDrag( args ) end
	self.MouseMove = function( sender, args ) 	self:_Drag( args ) end
	self.MouseUp = function( sender, args )		self:_EndDrag() end
	self.MouseEnter = function( sender, args ) 	self.highlightBox:SetVisible( true ) end
	self.MouseLeave = function( sender, args )	self.highlightBox:SetVisible( false ) end

	target.SizeChanged = function( sender, args ) 
		self:SetWidth( self.target:GetWidth() );
		self.highlightBox:SetSize( self.target:GetSize() );
		self.highlightBoxDecoration:SetSize( self.target:GetSize() );
		self.label:SetSize( self:GetSize() );
	end
	
	target.PositionChanged = function( sender, args ) 
		self:SetPosition( self.target:GetLeft(), self.target:GetTop() - self:GetHeight() );
		self.highlightBox:SetPosition( self.target:GetPosition() );
	end
end

-- callback initiating drag action
function DragBar:_BeginDrag( pos )
	self.dragging = true;
	self.dragOrigX, self.dragOrigY = pos.X, pos.Y; -- save the original position of the mouse pointer 
	self:SetBackground( rs_dragbar_depressed );
end

-- callback handling drag movements & repositioning
function DragBar:_Drag( pos )
	if not self.dragging then return end

	-- compute new position
	local newX = self:GetLeft() + pos.X - self.dragOrigX;
	local newY = self:GetTop() + pos.Y - self.dragOrigY;

	-- make sure the window stays entirely within the drawing area
	newX = Equendil.Utils.Maths.constrain( 0, newX, Turbine.UI.Display:GetWidth() - self:GetWidth() ); 
	newY = Equendil.Utils.Maths.constrain( 0, newY, Turbine.UI.Display:GetHeight() - self:GetHeight() - self.target:GetHeight() );
		
	-- move windows around accordingly
	self:SetPosition( newX, newY );
	self.target:SetPosition( newX, newY + self:GetHeight() );
	self.highlightBox:SetPosition( newX, newY + self:GetHeight() );
end

-- callback ending drag action
function DragBar:_EndDrag()
	self.dragging = false;
	self.Repositioned(); -- notify position changed
	self:SetBackground( rs_dragbar_normal );
end
